package pjvukol;

import org.junit.Test;
import static org.junit.Assert.*;

public class SlozenyIntervalTest {

    @Test
    public void testPridejLehci(){
        SlozenyInterval a = new SlozenyInterval(0,1);
        assertEquals(a.intervaly.size(), 1);

        a.pridej(new Interval(2,3));
        assertEquals(a.intervaly.size(), 2);

        a.pridej(new Interval(2,3));
        assertEquals(a.intervaly.size(), 2);

        a.pridej(new Interval(0,1));
        assertEquals(a.intervaly.size(), 2);

        a.pridej(new Interval(4,5));
        assertEquals(a.intervaly.size(), 3);

        a.pridej(new Interval(4,5));
        assertEquals(a.intervaly.size(), 3);

    }

    @Test
    public void testObsahuje() {
        SlozenyInterval a = new SlozenyInterval(0,1);
        // <<0,1>>
        assertTrue(a.obsahuje(0));
        assertTrue(a.obsahuje(1));
        assertTrue(a.obsahuje(0.5));
        assertFalse(a.obsahuje(1.1));
        assertFalse(a.obsahuje(-0.00001));

        a.pridej(new Interval(1,2));
        // <<0,2>>
        assertTrue(a.obsahuje(0));
        assertTrue(a.obsahuje(1));
        assertTrue(a.obsahuje(0.5));
        assertTrue(a.obsahuje(1.1));
        assertFalse(a.obsahuje(-0.00001));
        assertTrue(a.obsahuje(2));
        assertFalse(a.obsahuje(2.000001));

        a.pridej(3,4);
        // <<0,2><3,4>>
        assertTrue(a.obsahuje(0));
        assertTrue(a.obsahuje(2));
        assertFalse(a.obsahuje(2.0001));
        assertFalse(a.obsahuje(2.99999));
        assertTrue (a.obsahuje (3.5));
        assertFalse(a.obsahuje(4.1));

        a.pridej(5,6);
        // <<0;2><3;4><5;6>>
        assertTrue(a.obsahuje(0));
        assertTrue(a.obsahuje(2));
        assertFalse(a.obsahuje(2.0001));
        assertFalse(a.obsahuje(2.99999));
        assertTrue (a.obsahuje (3.5));
        assertFalse(a.obsahuje(4.1));
        assertTrue(a.obsahuje(5.5));

        a.pridej(7,8);
        // <<0;2><3;4><5;6><7;8>>
        assertTrue(a.obsahuje(0));
        assertTrue(a.obsahuje(2));
        assertFalse(a.obsahuje(2.0001));
        assertFalse(a.obsahuje(2.99999));
        assertTrue (a.obsahuje (3.5));
        assertFalse(a.obsahuje(4.1));
        assertTrue(a.obsahuje(5.5));
        assertTrue(a.obsahuje(7));
        assertTrue(a.obsahuje(7.5));

        a.pridej(6.5,8);
        // <<0;2><3;4><5;6><6,5;8>>
        assertTrue(a.obsahuje(6.5));
        assertTrue(a.obsahuje(8));
        assertTrue(a.obsahuje(7));
        assertFalse(a.obsahuje(4.5));
        assertFalse(a.obsahuje(6.4999999));

        a.pridej(3.5,5.5);
        // <<0;2><3;6><6,5;8>>
        assertTrue(a.obsahuje(4.1));
        assertTrue(a.obsahuje(5.6));
        assertTrue(a.obsahuje(3.4));
        assertFalse(a.obsahuje(6.0001));
        assertFalse(a.obsahuje(2.1));

        a.pridej(1,6.5);
        // <<0;8>>
        assertTrue(a.obsahuje(6.0001));
        assertTrue(a.obsahuje(2.1));
        assertFalse(a.obsahuje(8.001));

        a.pridej(0.5,10);
        // <<0;10>>
        assertTrue(a.obsahuje(8.001));

        a.pridej(11,20);
        // <<0;10><11;20>>
        assertFalse(a.obsahuje(10.5));

        a.pridej(10.5,10.6);
        // <<0;10><10,5;10,6><11;20>>
        assertTrue(a.obsahuje(10.5));
        assertFalse(a.obsahuje(10.4999));
        assertTrue(a.obsahuje(10));

        a.pridej(10.5,10.6);
        // <<0;10><10,5;10,6><11;20>>
        assertTrue(a.obsahuje(10.5));
        assertFalse(a.obsahuje(10.4999));
        assertFalse(a.obsahuje(10.601));
        assertTrue(a.obsahuje(10));

        a.pridej(10.6,11);
        // <<0;10><10;20>>
        assertFalse(a.obsahuje(10.4999));
        assertTrue(a.obsahuje(10.601));

        a.pridej(10,10.5);
        // <<0;20>>
        assertTrue(a.obsahuje(10.49999));
    }

    @Test
    public void testSjednoceni(){
        SlozenyInterval a = new SlozenyInterval(0,1);
        a.pridej(3,4);
        // <<0;1><3;4>>
        assertTrue(a.obsahuje(0));
        assertTrue(a.obsahuje(1));
        assertTrue(a.obsahuje(3));
        assertTrue(a.obsahuje(4));
        assertTrue(a.obsahuje(0.5));
        assertTrue(a.obsahuje(3.5));
        assertFalse(a.obsahuje(1.1));
        assertFalse(a.obsahuje(2));
        assertFalse(a.obsahuje(2.5));
        assertFalse(a.obsahuje(4.5));

        SlozenyInterval b = new SlozenyInterval(1,2);
        SlozenyInterval novy = a.sjednoceni(b);
        // <<0.0,2.0><3.0,4.0>>
        assertTrue(novy.obsahuje(0));
        assertTrue(novy.obsahuje(1));
        assertTrue(novy.obsahuje(3));
        assertTrue(novy.obsahuje(4));
        assertTrue(novy.obsahuje(0.5));
        assertTrue(novy.obsahuje(3.5));
        assertTrue(novy.obsahuje(1.1));
        assertTrue(novy.obsahuje(2));
        assertFalse(novy.obsahuje(2.5));
        assertFalse(novy.obsahuje(4.5));
        assertTrue(a.obsahuje(0));
        assertTrue(a.obsahuje(1));
        assertTrue(a.obsahuje(3));
        assertTrue(a.obsahuje(4));
        assertTrue(a.obsahuje(0.5));
        assertTrue(a.obsahuje(3.5));
        assertFalse(a.obsahuje(1.1));
        assertFalse(a.obsahuje(2));
        assertFalse(a.obsahuje(2.5));
        assertFalse(a.obsahuje(4.5));

        SlozenyInterval c = new SlozenyInterval(2,3);
        novy = novy.sjednoceni(c);
        // <<0.0,4.0>>
        assertTrue(novy.obsahuje(0));
        assertTrue(novy.obsahuje(1));
        assertTrue(novy.obsahuje(3));
        assertTrue(novy.obsahuje(4));
        assertTrue(novy.obsahuje(0.5));
        assertTrue(novy.obsahuje(3.5));
        assertTrue(novy.obsahuje(1.1));
        assertTrue(novy.obsahuje(2));
        assertTrue(novy.obsahuje(2.5));
        assertFalse(novy.obsahuje(4.5));
        assertTrue(a.obsahuje(0));
        assertTrue(a.obsahuje(1));
        assertTrue(a.obsahuje(3));
        assertTrue(a.obsahuje(4));
        assertTrue(a.obsahuje(0.5));
        assertTrue(a.obsahuje(3.5));
        assertFalse(a.obsahuje(1.1));
        assertFalse(a.obsahuje(2));
        assertFalse(a.obsahuje(2.5));
        assertFalse(a.obsahuje(4.5));
    }

    @Test
    public void testPrunikJeden(){
        SlozenyInterval a = new SlozenyInterval(0,1);
        a.pridej(3,4);
        a.pridej(5,6);
        // <<0;1><3;4><5;6>>

        Interval b = new Interval(0.5,2);
        SlozenyInterval novy = a.prunik(b);
        // <<0.5,1.0>>
        assertTrue(novy.obsahuje(0.5));
        assertTrue(novy.obsahuje(0.7));
        assertTrue(novy.obsahuje(1));
        assertFalse(novy.obsahuje(0.4));
        assertFalse(novy.obsahuje(3.5));
        assertFalse(novy.obsahuje(5.5));
        assertFalse(novy.obsahuje(6));
        assertFalse(novy.obsahuje(1.5));

        b = new Interval(0.5,3.5);
        novy = a.prunik(b);
        // <<0.5,1.0><3.0,3.5>>
        assertTrue(novy.obsahuje(0.5));
        assertTrue(novy.obsahuje(0.7));
        assertTrue(novy.obsahuje(1));
        assertFalse(novy.obsahuje(0.4));
        assertTrue(novy.obsahuje(3.5));
        assertFalse(novy.obsahuje(5.5));
        assertFalse(novy.obsahuje(6));
        assertFalse(novy.obsahuje(1.5));
        assertFalse(novy.obsahuje(2));
        assertTrue(novy.obsahuje(3));
    }


    @Test
    public void testPrunikSlozeny(){
        SlozenyInterval a = new SlozenyInterval(0,1);
        a.pridej(3,4);
        a.pridej(5,6);

        SlozenyInterval b = new SlozenyInterval(0.5,3);
        b.pridej(3.5,4);
        b.pridej(5.9,8);
        b.pridej(10,12);

        SlozenyInterval c = a.prunik(b);
        //<<0.5,1><3,3><3.5,4><5.9,6>>
        assertTrue(c.obsahuje(0.5));
        assertTrue(c.obsahuje(0.6));
        assertTrue(c.obsahuje(1));
        assertTrue(c.obsahuje(3));
        assertTrue(c.obsahuje(3.5));
        assertTrue(c.obsahuje(3.6));
        assertTrue(c.obsahuje(4));
        assertTrue(c.obsahuje(5.9));
        assertTrue(c.obsahuje(5.999));
        assertTrue(c.obsahuje(6.0));
        assertFalse(c.obsahuje(0.4));
        assertFalse(c.obsahuje(3.0001));
        assertFalse(c.obsahuje(5));
        assertFalse(c.obsahuje(5.8));
        assertFalse(c.obsahuje(8));
        assertFalse(c.obsahuje(11));
        assertFalse(c.obsahuje(3.2));
        assertFalse(c.obsahuje(4.01));
    }

}