package pjvukol;

import org.junit.Test;
import static org.junit.Assert.*;

/**
 * Tezsi varianta testu pro slozeny interval
 * @author Martin Plicka
 */
public class SlozenyIntervalTezsiTest {

    @Test
    public void testPridej() {
        SlozenyInterval a = new SlozenyInterval(0,1);
        assertEquals("<<0.0,1.0>>", a.toString());

        a.pridej(new Interval(1,2));
        assertEquals("<<0.0,2.0>>", a.toString());

        a.pridej(3,4);
        assertEquals("<<0.0,2.0><3.0,4.0>>", a.toString());

        a.pridej(5,6);
        assertEquals("<<0.0,2.0><3.0,4.0><5.0,6.0>>", a.toString());

        a.pridej(7,8);
        assertEquals("<<0.0,2.0><3.0,4.0><5.0,6.0><7.0,8.0>>", a.toString());

        a.pridej(6.5,8);
        assertEquals("<<0.0,2.0><3.0,4.0><5.0,6.0><6.5,8.0>>", a.toString());

        a.pridej(3.5,5.5);
        assertEquals("<<0.0,2.0><3.0,6.0><6.5,8.0>>", a.toString());

        a.pridej(1,6.5);
        assertEquals("<<0.0,8.0>>", a.toString());

        a.pridej(0.5,10);
        assertEquals("<<0.0,10.0>>", a.toString());

        a.pridej(11,20);
        assertEquals("<<0.0,10.0><11.0,20.0>>", a.toString());

        a.pridej(10.5,10.6);
        assertEquals("<<0.0,10.0><10.5,10.6><11.0,20.0>>", a.toString());

        a.pridej(10.5,10.6);
        assertEquals("<<0.0,10.0><10.5,10.6><11.0,20.0>>", a.toString());

        a.pridej(10.6,11);
        assertEquals("<<0.0,10.0><10.5,20.0>>", a.toString());
        
        a.pridej(10,10.5);
        assertEquals("<<0.0,20.0>>", a.toString());
    }

    @Test
    public void testSjednoceni(){
        SlozenyInterval a = new SlozenyInterval(0,1);
        // <<0;1>>
        assertEquals("<<0.0,1.0>>", a.toString());

        a.pridej(3,4);
        // <<0;1><3;4>>
        assertEquals("<<0.0,1.0><3.0,4.0>>", a.toString());

        SlozenyInterval b = new SlozenyInterval(1,2);
        SlozenyInterval novy = a.sjednoceni(b);
        assertEquals("<<0.0,1.0><3.0,4.0>>", a.toString());
        assertEquals("<<0.0,2.0><3.0,4.0>>", novy.toString());

        SlozenyInterval c = new SlozenyInterval(2,3);
        novy = novy.sjednoceni(c);
        assertEquals("<<0.0,1.0><3.0,4.0>>", a.toString());
        assertEquals("<<0.0,4.0>>", novy.toString());
    }

    @Test
    public void testPrunikJeden(){
        SlozenyInterval a = new SlozenyInterval(0,1);
        a.pridej(3,3.5);
        a.pridej(3.5,4);
        a.pridej(5,6);
        // <<0;1><3;4><5;6>>
        assertEquals("<<0.0,1.0><3.0,4.0><5.0,6.0>>", a.toString());

        Interval b = new Interval(0.5,2);
        SlozenyInterval novy = a.prunik(b);
        assertEquals("<<0.5,1.0>>", novy.toString());

        b = new Interval(0.5,3.5);
        novy = a.prunik(b);
        assertEquals("<<0.5,1.0><3.0,3.5>>", novy.toString());
    }


    @Test
    public void testPrunikSlozeny(){
        SlozenyInterval a = new SlozenyInterval(0,1);
        a.pridej(3,3.5);
        a.pridej(3.5,4);
        a.pridej(5,6);

        SlozenyInterval b = new SlozenyInterval(0.5,3);
        b.pridej(3.5,4);
        b.pridej(5.9,8);
        b.pridej(10,12);

        SlozenyInterval c = a.prunik(b);
        //<<0.5,1><3,3><3.5,4><5.9,6>>
        assertEquals("<<0.5,1.0><3.0,3.0><3.5,4.0><5.9,6.0>>",c.toString());
    }
}